

###MARIA BAS - February 2021 - Contact: maria.bas@udg.edu

##CONDUCTIVITY - SALINITY CONVERSION & OXYGEN CORRECTIONS

################################# FIRST STEP ###############################################

##First step to convert specific conductivity into salinity for the oxygen correction

##file (csv) with specific conductivity values. The name of the variable (column) should be called "conduct". Also must contain
##a variable called "datetime" in format: aaaa-mm-dd hh:mm:ss

conductivity <- read.csv("C:/Users/maria/Desktop/Maria_dell/Doctorat/DADES/prova2.csv", sep=",") ##keep an eye on changing the source!

conductivity$datetime <- as.POSIXct(conductivity$datetime,format="%Y-%m-%d %H:%M:%S",tz="UTC")
str(conductivity)
conduct_mS <- conductivity$conduct

#if we need to convert from mS/cm (CTD-diver measurements) to uS/cm (calculations)
conduct <- conduct_mS*1000

##set up a file with reference temperature values or 25 by default below
ref_temp <- rep(25, length(conductivity$conduct))

condKCl <- 29035.1640851+861.302764*ref_temp+4.6636947*(ref_temp^2)-0.0267243*(ref_temp^3)
Rt <- +1*(conduct/condKCl)
S1 <- 0.008-0.1692*(Rt^0.5)+25.3851*Rt+14.0941*(Rt^1.5)-7.0261*(Rt^2)+2.7081*(Rt^2.5)
S2 <- 0.0005-0.0056*(Rt^0.5)-0.0066*Rt-0.0375*(Rt^1.5)+0.0636*(Rt^2)-0.0144*(Rt^2.5)
S3 <- +1*(ref_temp-15)/(1+(0.0162*(ref_temp-15)))
salinity <- ifelse(conduct == 0, 0, S1+(S2*S3))


###################################ATENTION####################################
##DO NOT run twice ##here is doing the same as above but practicing loops
for(i in 1:length(conduct)){
  if (conduct[i] == 0){
    salinity[i] <- 0
  }else{
    salinity[i] <- S1[i]+(S2[i]*S3[i])
    }
}


#joining in the same file (data_results) the following variables: datetime, conductivity & salinity
data_results <- cbind(conductivity, salinity)
View(data_results)




################################# SECOND STEP###############################################

#salinity and atmospheric pressure correccions on dissolved oxygen concentration

#"data" file (csv) must contain the following variables (before running the function be sure all the variables have the same
#lenght and there is no NA values): datetime (format: aaaa-mm-dd hh:mm:ss), atmo.pressure (in atm), satdo (in %), temp (in ?C). 
#File containing all those variables should be named as "data" and it can be imported as follows.

data<-read.csv("C:/Users/maria/Desktop/Maria_dell/Doctorat/DADES/prova.csv", sep=",") ##keep an eye on changing the source!
data$datetime <- as.POSIXct(data$datetime,format="%Y-%m-%d %H:%M:%S",tz="UTC")
str(data)
View(data)

##Merging this file with the one obtained above containing the salinity values

data <- merge(data, data_results, all.x= TRUE, all.y = TRUE)

View(data)

salinity.correction <- function(data){
  
  atmo.pressure = data[['atmo.pressure']]
  satdo = data[['satdo']]
  temp = data[['temp']]
  salinity = data[['salinity']]
  
  #temperature conversion to kelvin to keep on working
  kelvin = 273.15 + temp
  
  # The salinity correction for the solubility of DO at a given salinity is (Benson and Krause 1984; APHA 2005)
  
  S1 <- 157570.1 / kelvin
  S2 <- -6.6423080E7 / (kelvin^2)
  S3 <- 1.2438E10 / (kelvin^3)
  S4 <-  -8.621949E11 / (kelvin^4)
  sal.factor <- -1.0*salinity*(0.017674-10.754/kelvin+2140.7/(kelvin^2))
  
  DOsalinity.corr <-exp(-139.34411+S1+S2+S3+S4+sal.factor)
  
  #Correction for atmospheric pressure
  
  #alpha <- 0.000975-0.00001426*kelvin+0.00000006436*(kelvin^2)
  
  #beta <- exp(11.8571-3840.7/kelvin-216961/(kelvin^2))
  
  #gamma <- ((1-beta/atmo.pressure) / (1-beta)) * ((1-alpha*atmo.pressure)/(1-alpha))
  
  #DO.sat refers to the solubility of DO to the temperature and salinity supplied at equilibrium (100%sat)
  DO.sat <- DOsalinity.corr #*atmo.pressure*gamma
  
  ##DO.corr refers to the real concentration of the sample with the temperature and salinity supplied
  DO.corr <- DOsalinity.corr * satdo /100

return(DO.corr)
  
} 
  
DO.corr <- salinity.correction(data)
write.csv(DO.corr, "DO.corr.csv") ##it will create a csv with the results of interest (DO values corrected).
